package MyFlow;

use strict;
use Socket;
use POSIX;
use Time::Local;

use vars qw (	@EXPORT @ISA @EXPORT_OK @VERSION
		$FDIR $FETC $FCFG %PN2T
		);

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(FlowDir FlowDate NSLookUp IP2STR STR2IP LogIt LOAD_CFG
		load_net inside_net load_list
		time2epoch fp2epoch timediff
		$FDIR $FETC $FCFG %PN2T
		);

@EXPORT = @EXPORT_OK;
@VERSION = '0.90';

### UpdateHere ###
$FDIR = "/usr/local/flow-scan";

### No need to be modified below ###
$FETC = "$FDIR/etc";
$FCFG = "$FETC/flow-scan.txt";
%PN2T = ( "1" => "ICMP", "6" => "TCP", "17" => "UDP");

sub FlowDir {
        my $now = shift;

        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
                localtime($now);

	$year = sprintf "%04d", 1900+$year;
	$mon = sprintf "%02d", $mon+1;
	$mday = sprintf "%02d", $mday;

        return "$year/$year-$mon/$year-$mon-$mday";
}

sub FlowDate {
        my $now = shift;

        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);

        ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
                localtime($now);

	$year = sprintf "%04d", 1900+$year;
	$mon = sprintf "%02d", $mon+1;
	$mday = sprintf "%02d", $mday;

        return "$year-$mon-$mday";
}

sub IP2STR {
	my ($ip) = shift;

	my ($ip1, $ip2, $ip3, $ip4) = $ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;
	my $str = sprintf "IP_%s_%s_%s_%s", $ip1, $ip2, $ip3, $ip4;

	return $str;
}

sub STR2IP {
	my ($str) = shift;

	my ($ip1, $ip2, $ip3, $ip4) = $str =~ /IP_(\d+)_(\d+)_(\d+)_(\d+)/;
	my $ip = sprintf "%s.%s.%s.%s", $ip1, $ip2, $ip3, $ip4;

	return $ip;
}

sub NSLookUp
{
	my $addr = shift;
	my $FQDN;

	open (NS ,"/usr/sbin/nslookup $addr 2>&1 |");
	while (<NS>) {
		if (/^Name/) {
			($FQDN) = /Name:\s+(\S+)/;
		}
	}
	close(NS);

	if (! defined $FQDN) {
		$FQDN = "U";
	}

	return $FQDN;
}

sub LogIt {
	my ($fn,$msg) = @_;

	my $now = localtime;

	open (LOG, ">>$fn");
	printf "> $now : $msg\n";
	printf LOG "$now : $msg\n";
	close(LOG);
}

sub LOAD_CFG {
        my ($ref) = @_;
	my ($key,$val);

        open(CFG, "$FCFG");
        while (<CFG>) {
                chomp;
                next if (/^$/);
                next if (/^#/);
		($key, $val) = /(\S+) = (.+)$/;
		$$ref{$key} = eval($val);
#		printf "%s => %s\n", $key, $$ref{$key};
        }
        close(CFG);
}

sub load_net {
    my ($fn,$r_net,$r_mask) = @_;
    my ($net,$mask);

    open(IN, "$fn");
    while (<IN>) {
        chomp;
        ($net,$mask) = split /\//;
        $$r_net{"$net-$mask"}= dottedquad2long($net);
        $$r_mask{"$net-$mask"} = makemask($mask);
    }
    close(IN);
}

sub inside_net {
    my ($ip,$r_net,$r_mask) = @_;
    my ($key,$net1,$net2);
    my $yes = 0;

    foreach $key (keys %$r_net) {
        $net1 = dottedquad2long($ip)&$$r_mask{$key};
        $net2 = $$r_net{$key}&$$r_mask{$key};
        if ($net1 == $net2) {
           $yes = 1;
           last;
        }
    }
    return $yes;
}

sub dottedquad2long {
#  return unpack('N', inet_aton(shift));
#  or
	return unpack 'N', pack 'C4', split /\./, shift;
#  or
#  my @seg = split(/\./, shift);
#  return $seg[0]*256**3+$seg[1]*256**2+$seg[2]*256+$seg[3];
}

sub makemask {
  my $mask_length = shift;
  my $binary = '1' x $mask_length . '0' x (32 - $mask_length);
  return bin2long($binary);
}

sub bin2long {
  return unpack('N', pack('B32', shift));
}

sub load_list {
	my ($fn,$ref) = @_;

	open(IN, "$fn");
	while (<IN>) {
		chomp;
		next if (/^#/);
		next if (/^$/);
		$$ref{$_} = 1;
	}
	close(IN)
}

sub time2epoch {
	my $tmstr = shift;
	my $epoch;

#Sun Oct 27 14:50:01 2002
	my %MTAB = ('Jan',0,'Feb',1,'Mar',2,'Apr',3,'May',4,'Jun',5,
			'Jul',6,'Aug',7,'Sep',8,'Oct',9,'Nov',10,'Dec',11);
	my ($mo,$dd,$hh,$mm,$ss,$yy);

	($mo,$dd,$hh,$mm,$ss,$yy) = $tmstr
		=~ /\S+\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\d+)/;

	$mo = $MTAB{$mo};
	$yy = $yy - 1900;

	$epoch = timelocal($ss,$mm,$hh,$dd,$mo,$yy);

	return $epoch;
}

sub fp2epoch {
	my $fpstr = shift;
	my $epoch;

#2003.0101.00:00:00.000

	my ($yy,$mo,$dd,$hh,$mm,$ss) = $fpstr
		=~ /(\d+)\.(\d\d)(\d\d)\.(\d+):(\d+):(\d+)\.\d+/;

	$mo--;
	$yy = $yy - 1900;

	$epoch = timelocal($ss,$mm,$hh,$dd,$mo,$yy);

	return $epoch;
}

sub timediff {
	my ($t0,$t1) = @_;

	my ($e0,$e1,$diff);
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) 
		= localtime(time);
	my ($mo,$dd,$hh,$mm,$ss);

	($mo,$dd,$hh,$mm,$ss) = $t0 =~ /(\d\d)(\d\d)\.(\d+):(\d+):(\d+)\.\d+/;
	$mo--;
	$e0 = timelocal($ss,$mm,$hh,$dd,$mo,$year);

	($mo,$dd,$hh,$mm,$ss) = $t1 =~ /(\d\d)(\d\d)\.(\d+):(\d+):(\d+)\.\d+/;
	$mo--;
	$e1 = timelocal($ss,$mm,$hh,$dd,$mo,$year);

	$diff = $e1 - $e0;

	return $diff;
}

