package MyScore;

use strict;

use vars qw(@EXPORT @ISA @EXPORT_OK @VERSION);

require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(score_host_scan score_port_scan load_ipv4_reserved);
@EXPORT = @EXPORT_OK;
@VERSION = '0.90';

sub score_host_scan {
	my ($ref_check,$ref_reserved) = @_;

	my $GAP = 5;
	my $BAD_IP_COST = 10;
	my $SCAN_HOST_COST = 1;
	my $SAME_FLOW_COST = 1;
	my $HTTP_SYNC_COST = 1;
	my $SWEAP_IP_COST = 1;
	my ($dst,$dp,$pt,$packet,$octet);
	my ($dp2,$pt2,$packet2,$octet2) = (0,0,0,0);
        my ($n1,$n2,$n3,$n4) = (0,0,0,0);
	my ($o1,$o2,$o3,$o4) = (0,0,0,0);
	my ($score,$badip,$scan) = (0,0,0);
	my %HOST = ();
	my $line;

	### sort before checking ###
	foreach $line (sort @$ref_check) {
		($dst,$dp,$pt,$packet,$octet) = split /,/, $line;
		($n1,$n2,$n3,$n4) = split /\./, $dst;

		### IANA Reserved IP ###
                if (defined $$ref_reserved{"$n1"}) {
			$score += $BAD_IP_COST;
			$badip++;
		}

		### Host Scan ##
		if (!defined $HOST{$dst}) {
			$HOST{$dst} = 1;
			$scan++;
			$score += $SCAN_HOST_COST;
		}

		### SAME FLOW ###
		if (("$pt" eq "$pt2") && ("$dp" eq "$dp2") && ("$packet" eq "$packet2") && ("$octet" eq "$octet2")) {
			$score += $SAME_FLOW_COST;
		}

		### HTTP SYNC ###
		if (("$pt" eq "6") && ($packet <= 3) && ($octet/$packet == 48)) {
			$score += $HTTP_SYNC_COST;
		}

		### SLAPPER ###
		if ("$n1.$n2.$n3" eq "$o1.$o2.$o3") {
			if (abs($n4 - $o4) <= $GAP) {
				$score += $SWEAP_IP_COST;
			}
		}

		### SPIDA ###
		elsif ("$n1.$n2.$n4" eq "$o1.$o2.$o4") {
			if (abs($n3 - $o3) <= $GAP) {
				$score += $SWEAP_IP_COST;
			}
		}

		### Iraqi ###
		elsif ("$n2.$n3.$n4" eq "$o2.$o3.$o4") {
			if (abs($n1 - $o1) <= $GAP) {
				$score += $SWEAP_IP_COST;
			}
		}

		### Not Found ###
		elsif ("$n1.$n3.$n4" eq "$o1.$o3.$o4") {
			if (abs($n2 - $o2) <= $GAP) {
				$score += $SWEAP_IP_COST;
			}
		}

		($o1,$o2,$o3,$o4) = ($n1,$n2,$n3,$n4);
		($pt2,$dp2,$packet2,$octet2) = ($pt,$dp,$packet,$octet);
	}

	return ($score,$badip,$scan);
}

sub score_port_scan {
	my ($ref_check,$ref_reserved) = @_;

	my $GAP = 5;;
	my $BAD_IP_COST = 10;
	my $SCAN_PORT_COST = 1;
	my $SWEAP_PORT_COST = 1;
	my ($dst,$dp,$pt,$packet,$octet);
	my ($n1,$dp2) = (0,0);
	my ($score,$badip,$scan) = (0,0,0);
	my %PORT = ();
	my $line;

	### sort before checking ###
	foreach $line (sort @$ref_check) {
		($dst,$dp,$pt,$packet,$octet) = split /,/, $line;
		($n1) = split /\./, $dst;

                ### IANA Reserved IP ###
                if (defined $$ref_reserved{$n1}) {
			$score += $BAD_IP_COST;
			$badip++;
		}

		if (! defined $PORT{$dp}) {
			$PORT{$dp} = 1;
			$scan++;
			$score += $SCAN_PORT_COST;
		}

		if ( $dp2 == $dp ) {
			next;
		}
		elsif ( abs($dp2 - $dp) <= $GAP ) {
			$score += $SWEAP_PORT_COST ;
		}
		$dp2 = $dp;
	}
	return ($score,$badip,$scan);
}

sub load_ipv4_reserved {
	my ($ref_reserved,$reserved) = @_;
	my ($net,$com,$ip);

	open (IN,$reserved);
	while (<IN>) {
		next if (! /^\d/);
		($net,$com) = /^(\S+)\s+(.+)$/;
		if ($com =~ /IANA - (Multicast|Reserved)/) {
			($ip) = $net =~ /(\d+)\/\d+/;
			$ip = $ip * 1;
			$$ref_reserved{$ip} = 1;
		}
	}
	close (IN);
}

